@@ -3,16 +3,12 @@
 // Purpose:
 // Date: 08/11/1999
 // Author: Guilhem Lavaux <lavaux@easynet.fr> (C) 1999
-// CVSID: $Id: sndesd.h,v 1.4 2005/08/02 21:44:34 MW Exp $
+// CVSID: $Id: sndesd.h,v 1.5 2005/09/23 12:47:04 MR Exp $
 // License:    wxWindows license
 // --------------------------------------------------------------------------
 #ifndef _WX_SNDESD_H
 #define _WX_SNDESD_H
 
-#ifdef __GNUG__
-#pragma interface "sndesd.h"
-#endif
-
 #include "wx/defs.h"
 #include "wx/string.h"
 #include "wx/mmedia/defs.h"
