@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: treetest.cpp,v 1.111 2007/01/07 15:43:22 VZ Exp $
+// RCS-ID:      $Id: treetest.cpp,v 1.111.2.1 2007/03/11 23:29:26 PC Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -352,14 +352,20 @@ void MyFrame::OnIdle(wxIdleEvent& event)
     if ( m_treeCtrl )
     {
         wxTreeItemId idRoot = m_treeCtrl->GetRootItem();
+        wxString status;
+        if (idRoot.IsOk())
+        {
+            wxTreeItemId idLast = m_treeCtrl->GetLastChild(idRoot);
+            status = wxString::Format(
+                _T("Root/last item is %svisible/%svisible"),
+                m_treeCtrl->IsVisible(idRoot) ? _T("") : _T("not "),
+                idLast.IsOk() && m_treeCtrl->IsVisible(idLast)
+                    ? _T("") : _T("not "));
+        }
+        else
+            status = _T("No root item");
 
-        SetStatusText(wxString::Format
-                      (
-                        _T("Root/last item is %svisible/%svisible"),
-                        m_treeCtrl->IsVisible(idRoot) ? _T("") : _T("not "),
-                        m_treeCtrl->IsVisible(m_treeCtrl->GetLastChild(idRoot))
-                            ? _T("") : _T("not ")
-                      ), 1);
+        SetStatusText(status, 1);
     }
 #endif // wxUSE_STATUSBAR
 
