@@ -4,7 +4,7 @@
 // Author:      Hans Van Leemputten
 // Modified by: Benjamin I. Williams / Kirix Corporation
 // Created:     29/07/2002
-// RCS-ID:      $Id: tabmdi.cpp,v 1.24 2007/01/06 08:59:01 RD Exp $
+// RCS-ID:      $Id: tabmdi.cpp,v 1.24.2.2 2007/03/12 13:49:11 BIW Exp $
 // Copyright:   (c) Hans Van Leemputten
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -177,7 +177,10 @@ void wxAuiMDIParentFrame::SetChildMenuBar(wxAuiMDIChildFrame* pChild)
     if (!pChild)
     {
         // No Child, set Our menu bar back.
-        SetMenuBar(m_pMyMenuBar);
+        if (m_pMyMenuBar)
+            SetMenuBar(m_pMyMenuBar);
+             else
+            SetMenuBar(GetMenuBar());
 
         // Make sure we know our menu bar is in use
         m_pMyMenuBar = NULL;
@@ -331,11 +334,6 @@ void wxAuiMDIParentFrame::DoHandleMenu(wxCommandEvent& event)
                 {
                     return; // failure
                 }
-                 else
-                {
-                    delete m_pActiveChild;
-                    m_pActiveChild = NULL;
-                }
             }
             break;
         case wxWINDOWNEXT:
