@@ -4,7 +4,7 @@
 // Author:      Jaakko Salli
 // Modified by:
 // Created:     Apr-30-2006
-// RCS-ID:      $Id: combocmn.cpp,v 1.56 2007/01/08 07:11:51 PC Exp $
+// RCS-ID:      $Id: combocmn.cpp,v 1.56.2.4 2007/04/16 20:51:27 RD Exp $
 // Copyright:   (c) 2005 Jaakko Salli
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -1855,7 +1855,7 @@ void wxComboCtrlBase::ShowPopup()
     }
 
     winPopup->Enable();
-    
+
     wxASSERT( !m_popup || m_popup == popup ); // Consistency check.
 
     wxSize adjustedSize = m_popupInterface->GetAdjustedSize(widthPopup,
@@ -2191,16 +2191,19 @@ void wxComboCtrlBase::SetValueWithEvent(const wxString& value, bool withEvent)
             m_text->SelectAll();
     }
 
-    m_valueString = value;
-
-    Refresh();
-
     // Since wxComboPopup may want to paint the combo as well, we need
     // to set the string value here (as well as sometimes in ShowPopup).
-    if ( m_valueString != value && m_popupInterface )
+    if ( m_valueString != value )
     {
-        m_popupInterface->SetStringValue(value);
+        m_valueString = value;
+
+        EnsurePopupControl();
+
+        if (m_popupInterface)
+            m_popupInterface->SetStringValue(value);
     }
+
+    Refresh();
 }
 
 void wxComboCtrlBase::SetValue(const wxString& value)
