@@ -2,7 +2,7 @@
 // Name:        src/gtk/dataview.cpp
 // Purpose:     wxDataViewCtrl GTK+2 implementation
 // Author:      Robert Roebling
-// Id:          $Id: dataview.cpp,v 1.53 2006/12/04 23:05:04 VZ Exp $
+// Id:          $Id: dataview.cpp,v 1.53.4.1 2007/03/09 16:25:07 VZ Exp $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -810,7 +810,8 @@ bool wxGtkDataViewListModelNotifier::RowChanged( unsigned int row )
 bool wxGtkDataViewListModelNotifier::ValueChanged( unsigned int model_col, unsigned int model_row )
 {
     // This adds GTK+'s missing MVC logic for ValueChanged
-    wxNode *node = GetOwner()->m_viewingColumns.GetFirst();
+    wxObjectList::compatibility_iterator
+        node = GetOwner()->m_viewingColumns.GetFirst();
     while (node)
     {
         wxDataViewViewingColumn* viewing_column = (wxDataViewViewingColumn*) node->GetData();
@@ -847,7 +848,8 @@ bool wxGtkDataViewListModelNotifier::RowsReordered( unsigned int *new_order )
     gtk_tree_path_free (path);
 
     // This adds GTK+'s missing MVC logic for RowsReordered
-    wxNode *node = GetOwner()->m_viewingColumns.GetFirst();
+    wxObjectList::compatibility_iterator
+        node = GetOwner()->m_viewingColumns.GetFirst();
     while (node)
     {
         wxDataViewViewingColumn* viewing_column = (wxDataViewViewingColumn*) node->GetData();
