@@ -4,7 +4,7 @@
 //              GtkFixed. It makes use of the gravity window property and
 //              therefore does not work with GTK 1.0.
 // Author:      Robert Roebling
-// Id:          $Id: win_gtk.c,v 1.98 2006/10/20 05:47:15 PC Exp $
+// Id:          $Id: win_gtk.c,v 1.98.4.1 2007/03/07 06:17:23 PC Exp $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWidgets licence
 /////////////////////////////////////////////////////////////////////////// */
@@ -340,8 +340,12 @@ gtk_pizza_set_size (GtkPizza   *pizza,
 
         if (child->widget == widget)
         {
-            child->x = x;
-            child->y = y;
+            if (child->x != x || child->y != y)
+            {
+                child->x = x;
+                child->y = y;
+                gtk_widget_queue_resize(widget);
+            }
 
             gtk_widget_set_size_request (widget, width, height);
 
