@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Agron Selimaj
 // Created:     04/01/98
-// RCS-ID:      $Id: listctrl_mac.cpp,v 1.61 2007/01/08 02:52:09 KO Exp $
+// RCS-ID:      $Id: listctrl_mac.cpp,v 1.61.2.2 2007/05/10 02:02:12 VZ Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -654,6 +654,8 @@ void wxListCtrl::FireMouseEvent(wxEventType eventType, wxPoint position)
 
 void wxListCtrl::OnChar(wxKeyEvent& event)
 {
+
+    
     if (m_dbImpl)
     {
         wxListEvent le( wxEVT_COMMAND_LIST_KEY_DOWN, GetId() );
@@ -661,6 +663,35 @@ void wxListCtrl::OnChar(wxKeyEvent& event)
         le.m_code = event.GetKeyCode();
         le.m_itemIndex = -1;
         
+        if (m_current == -1)
+        {
+            // if m_current isn't set, check if there's been a selection
+            // made before continuing
+            m_current = GetNextItem(-1, wxLIST_NEXT_BELOW, wxLIST_STATE_SELECTED);
+        }
+        
+        // We need to determine m_current ourselves when navigation keys
+        // are used. Note that PAGEUP and PAGEDOWN do not alter the current
+        // item on native Mac ListCtrl, so we only handle up and down keys.
+        switch ( event.GetKeyCode() )
+        {
+            case WXK_UP:
+                if ( m_current > 0 )
+                    m_current -= 1;
+                else
+                    m_current = 0;
+                    
+                break;
+
+            case WXK_DOWN:
+                if ( m_current < GetItemCount() - 1 )
+                    m_current += 1;
+                else
+                    m_current = GetItemCount() - 1;
+                    
+                break;
+        }
+
         if (m_current != -1)
         {
             le.m_itemIndex = m_current;
@@ -1174,7 +1205,7 @@ bool wxListCtrl::SetItemState(long item, long state, long stateMask)
     if (m_dbImpl)
     {
         DataBrowserSetOption option = kDataBrowserItemsAdd;
-        if ( stateMask == wxLIST_STATE_SELECTED && state == 0 )
+        if ( (stateMask & wxLIST_STATE_SELECTED) && state == 0 )
             option = kDataBrowserItemsRemove;
 
         if (item == -1)
@@ -1202,8 +1233,15 @@ bool wxListCtrl::SetItemState(long item, long state, long stateMask)
             if ( HasFlag(wxLC_VIRTUAL) )
             {
                 long itemID = item+1;
+                bool isSelected = IsDataBrowserItemSelected(m_dbImpl->GetControlRef(), (DataBrowserItemID)itemID );
+                bool isSelectedState = (state == wxLIST_STATE_SELECTED);
+
+                // toggle the selection state if wxListInfo state and actual state don't match.
+                if ( (stateMask & wxLIST_STATE_SELECTED) && isSelected != isSelectedState )
+                {
                 SetDataBrowserSelectedItems(m_dbImpl->GetControlRef(), 1, (DataBrowserItemID*)&itemID, option);
             }
+            }
             else
             {
                 wxListItem info;
@@ -1288,7 +1326,7 @@ long wxListCtrl::GetItemData(long item) const
 }
 
 // Sets the item data
-bool wxListCtrl::SetItemData(long item, long data)
+bool wxListCtrl::SetItemPtrData(long item, wxUIntPtr data)
 {
     if (m_genericImpl)
         return m_genericImpl->SetItemData(item, data);
@@ -1302,6 +1340,11 @@ bool wxListCtrl::SetItemData(long item, long data)
     return SetItem(info);
 }
 
+bool wxListCtrl::SetItemData(long item, long data)
+{
+    return SetItemPtrData(item, data);
+}
+
 wxRect wxListCtrl::GetViewRect() const
 {
     wxASSERT_MSG( !HasFlag(wxLC_REPORT | wxLC_LIST),
@@ -1578,8 +1621,8 @@ long wxListCtrl::GetNextItem(long item, int geom, int state) const
                     return line;
             }
         }
-        
-        if ( geom == wxLIST_NEXT_ALL || geom == wxLIST_NEXT_ABOVE )
+
+        if ( geom == wxLIST_NEXT_ABOVE )
         {
             int item2 = item;
             if ( item2 == -1 )
