@@ -6,7 +6,7 @@
 # Author:       Robin Dunn
 #
 # Created:      6-March-2000
-# RCS-ID:       $Id: run.py,v 1.33 2007/01/17 23:39:00 RD Exp $
+# RCS-ID:       $Id: run.py,v 1.33.2.1 2007/03/08 19:24:13 RD Exp $
 # Copyright:    (c) 2000 by Total Control Software
 # Licence:      wxWindows license
 #----------------------------------------------------------------------------
@@ -18,7 +18,7 @@ on the command line.
 """
 
 import wx
-import wx.lib.mixins.inspect
+import wx.lib.mixins.inspection
 import sys, os
 
 # stuff for debugging
@@ -40,7 +40,7 @@ class Log:
     write = WriteText
 
 
-class RunDemoApp(wx.App, wx.lib.mixins.inspect.InspectionMixin):
+class RunDemoApp(wx.App, wx.lib.mixins.inspection.InspectionMixin):
     def __init__(self, name, module, useShell):
         self.name = name
         self.demoModule = module
