@@ -6,7 +6,7 @@
 # Author:       Pierre Hjlm (from C++ original by Julian Smart)
 #
 # Created:      2004-05-08
-# RCS-ID:       $Id: _lines.py,v 1.18 2007/01/12 18:40:51 RD Exp $
+# RCS-ID:       $Id: _lines.py,v 1.18.2.1 2007/02/16 23:32:37 RD Exp $
 # Copyright:    (c) 2004 Pierre Hjlm - 1998 Julian Smart
 # Licence:      wxWindows license
 #----------------------------------------------------------------------------
@@ -218,16 +218,6 @@ class LineShape(Shape):
         self._lineControlPoints = []
         self._arcArrows = []
 
-    def __del__(self):
-        if self._lineControlPoints:
-            self._lineControlPoints = []
-        for i in range(3):
-            if self._labelObjects[i]:
-                self._labelObjects[i].Select(False)
-                self._labelObjects[i].RemoveFromCanvas(self._canvas)
-        self._labelObjects = []
-        self.ClearArrowsAtPosition(-1)
-
     def GetFrom(self):
         """Return the 'from' object."""
         return self._from
@@ -492,6 +482,11 @@ class LineShape(Shape):
             self._from.GetLines().remove(self)
         self._to = None
         self._from = None
+        for i in range(3):
+            if self._labelObjects[i]:
+                self._labelObjects[i].Select(False)
+                self._labelObjects[i].RemoveFromCanvas(self._canvas)
+        self.ClearArrowsAtPosition(-1)
 
     def SetEnds(self, x1, y1, x2, y2):
         """Set the end positions of the line."""
