@@ -1,8 +1,8 @@
 """Base frame with menu."""
 
 __author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
-__cvsid__ = "$Id: frame.py,v 1.12 2007/01/12 19:20:27 RD Exp $"
-__revision__ = "$Revision: 1.12 $"[11:-2]
+__cvsid__ = "$Id: frame.py,v 1.12.2.1 2007/01/26 07:17:30 RD Exp $"
+__revision__ = "$Revision: 1.12.2.1 $"[11:-2]
 
 import wx
 import os
@@ -646,7 +646,7 @@ class Frame(wx.Frame):
 
 
     def LoadSettings(self, config):
-        """Called be derived classes to load settings specific to the Frame"""
+        """Called by derived classes to load settings specific to the Frame"""
         pos  = wx.Point(config.ReadInt('Window/PosX', -1),
                         config.ReadInt('Window/PosY', -1))
                         
